package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaCar;
import com.huiju.welcome.model.TaContentLib;
import com.huiju.welcome.service.ITaCarService;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-06-11
 */
@RestController
@RequestMapping("/")
public class TaCarController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCarController.class);

    @Autowired
    public ITaCarService iTaCarService;

    @PutMapping("/car/whitelist/{id}")
    public ResponseBean addWhiteList(@PathVariable String id) {
        iTaCarService.setCarType(id, CommConstant.TYPE_CAR_INNER);
        return ResponseBean.success("");
    }

    @DeleteMapping("/car/whitelist/{id}")
    public ResponseBean deleteWhiteList(@PathVariable String id) {
        iTaCarService.setCarType(id, CommConstant.TYPE_CAR_NORMAL);
        return ResponseBean.success("");
    }
    
    @ApiOperation(value = "分页获取车辆信息", notes = "分页获取车辆信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "plateNumber", value = "车牌号"),
    })
    @GetMapping("/cars")
    public ResponseBean getCarList(@RequestParam Integer pageNum, @RequestParam  Integer pageSize,@RequestParam String plateNumber){
        ResponseBean responseBean = new ResponseBean();
    
        Page<TaCar> pg = new Page<>();
        pg.setCurrent(pageNum);
        pg.setSize(pageSize);
        QueryWrapper<TaCar> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("create_date");
        queryWrapper.eq("status", StatusUtils.Normal);
        queryWrapper.like("plate_number",plateNumber);
        responseBean.addSuccess(iTaCarService.page(pg, queryWrapper));
        
        return responseBean;
    }

}
