package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaCarRecord;
import com.huiju.welcome.service.ITaCarRecordService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-06-11
 */
@RestController
@RequestMapping("/")
public class TaCarRecordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCarRecordController.class);

    @Autowired
    public ITaCarRecordService iTaCarRecordService;


    /**
     * 保存对象
     * @param taCarRecord 实体对象
     * @return
     */
    @RequestMapping(value="/carRecord",method= RequestMethod.POST)
    public ResponseBean save(@RequestBody TaCarRecord taCarRecord){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaCarRecordService.save(taCarRecord)){
                responseBean.addSuccess(taCarRecord);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCarRecordAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/carRecord/{id}", method= RequestMethod.DELETE)
    public ResponseBean delete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaCarRecordService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCarRecordDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param taCarRecord 实体对象
     * @return
     */
    @RequestMapping(value="/carRecord/{id}",method= RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaCarRecord taCarRecord, @PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {

            boolean success = iTaCarRecordService.updateRecordByMiniapp(taCarRecord);

            if (success){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCarRecordUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/carRecord/{id}",method= RequestMethod.GET)
    public ResponseBean get(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaCarRecordService.getById(id));
        }catch (Exception e){
            logger.error("taCarRecordGetByID -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 分页查询
     */

    @RequestMapping(value = "/carRecord", method = RequestMethod.GET)
    public ResponseBean list(@RequestParam(value = "plateNumber", required = false) String plateNumber,
                             @RequestParam(value = "recordDate", required = false) @DateTimeFormat(pattern = "yyyy-MM-dd") Date recordDate,
                             @RequestParam(value = "customerName", required = false) String customerName,
                             @RequestParam(value = "phone", required = false) String phone,
                             @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @RequestParam(value = "pageSize", defaultValue = "1") Integer pageSize) {
        ResponseBean responseBean = new ResponseBean();

        IPage<TaCarRecord> page = new Page<>();
        page.setCurrent(pageNum);
        page.setSize(pageSize);

        IPage<TaCarRecord> pageList = iTaCarRecordService.getPageList(page, plateNumber, recordDate, customerName, phone);
        responseBean.addSuccess(pageList);
        return responseBean;
    }

    @RequestMapping("/miniapp/carRecord/timeline")
    public ResponseBean timeline() {
        List<TaCarRecord> result = iTaCarRecordService.getTimeline();
        return ResponseBean.success(result);
    }
}
