package com.huiju.welcome.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaCustomerTransfer;
import com.huiju.welcome.model.TdGoodsType;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaCustomerTransferService;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-07-16
 */
@RestController
@RequestMapping("/")
public class TaCustomerTransferController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCustomerTransferController.class);

    @Autowired
    public ITaCustomerTransferService iTaCustomerTransferService;
    @Autowired
    public ITaCustomerService iTaCustomerService;


    /**
     * 保存对象
     * @param taCustomerTransfer 实体对象
     * @return
     */
    @RequestMapping(value="/taCustomerTransfer/add",method= RequestMethod.POST)
    public ResponseBean tpShopImgAdd(@RequestBody TaCustomerTransfer taCustomerTransfer){
        ResponseBean responseBean = new ResponseBean();
        try {
            taCustomerTransfer.setTransferTime(LocalDateTime.now());
            Boolean flag = iTaCustomerTransferService.save(taCustomerTransfer);
            Boolean flagUpdate = false;
            TaCustomer taCustomer = iTaCustomerService.getById(taCustomerTransfer.getCustomerId());
            taCustomer.setConsultantId(taCustomerTransfer.getToUserId());
            taCustomer.setRealtyConsultant(taCustomerTransfer.getToUserName());
            flagUpdate = iTaCustomerService.updateById(taCustomer);
            if (flag&&flagUpdate){
                responseBean.addSuccess(taCustomerTransfer);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCustomerTransferAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taCustomerTransfer/delete/{id}", method= RequestMethod.DELETE)
    public ResponseBean taCustomerTransferDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaCustomerTransferService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCustomerTransferDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param taCustomerTransfer 实体对象
     * @return
     */
    @RequestMapping(value="/taCustomerTransfer/update",method= RequestMethod.PUT)
    public ResponseBean tpShopImgUpdate(@RequestBody TaCustomerTransfer taCustomerTransfer){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaCustomerTransferService.updateById(taCustomerTransfer)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taCustomerTransferUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taCustomerTransfer/get/{id}",method= RequestMethod.GET)
    public ResponseBean tpShopImgGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaCustomerTransferService.getById(id));
        }catch (Exception e){
            logger.error("taCustomerTransferDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @ApiOperation(value = "人员流转记录", notes = "人员流转记录")
    @ApiImplicitParams({
    })
    @RequestMapping(value = "/taCustomerTransfer", method = RequestMethod.GET)
    public ResponseBean customerTransferList(@RequestParam(defaultValue = "1") int pageNum,
                                 @RequestParam(defaultValue = "10") int pageSize,
                                 @RequestParam(value = "fromUserName",required = false) String fromUserName,
                                             @RequestParam(value = "fromUserId",required = false) Integer fromUserId,
                                             @RequestParam(value = "toUserName",required = false) String toUserName,
                                             @RequestParam(value = "toUserId",required = false) Integer toUserId) {
        IPage<TaCustomerTransfer> pg = new Page<>(pageNum, pageSize);

        ResponseBean userManagement= iTaCustomerTransferService.customerTransferList(pg,fromUserName,fromUserId,toUserName,toUserId);

        return userManagement;
    }
}
