package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.*;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.MiniApp;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-12
 */
@Service
public class TaCustomerServiceImpl extends ServiceImpl<TaCustomerMapper, TaCustomer> implements ITaCustomerService {
    Logger log = LoggerFactory.getLogger(TaCustomerServiceImpl.class);

    @Autowired
    TaCustomerMapper taCustomerMapper;

    @Autowired
    TaCustomerMapMapper taCustomerMapMapper;

    @Autowired
    TaMainUsherRecordMapper taMainUsherRecordMapper;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaMainUsherRecordService iTaMainUsherRecordService;

    @Autowired
    ITaVisitorAppointmentService iTaVisitorAppointmentService;

    @Autowired
    ITaFollowupRecordService iTaFollowupRecordService;

    @Autowired
    TaWechatUserMapper taWechatUserMapper;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    ISysParamService iSysParamService;

    @Autowired
    MiniApp miniApp;
    
    @Autowired
    private ISysDictService iSysDictService;

    @Override
    public List<TaPerson> getGroup(Integer customerId) {
        return taCustomerMapMapper.getGroup(customerId);
    }

    @Override
    public Map<String, Object> getLastVisitInfo(Integer customerId) {
        return taMainUsherRecordMapper.getSummaryOfCustomer(customerId);
    }

    @Override
    public void judgeNewCust(TaPerson taPerson) throws Exception {
        // 首先判定这个人 2 小时内是否来访过
        Float interval = iSysParamService.getCustomerVisitInterval();
        LocalDateTime dt = LocalDateTime.now().minusMinutes(interval.intValue());
        List<TaMainUsherRecord> lastList = taMainUsherRecordMapper.getLastList(taPerson.getRealId(), dt);
        if (null != lastList && lastList.size() > 0) return;

        // 查询是否有映射客户 - 如果有多个映射客户, 先随机取一条
        // 暂时没有找到好的替代方案
        TaCustomer taCustomer = taCustomerMapper.getCustomerOf(taPerson.getPersonId(), taPerson.getRealId());

        // 如果是新人过来
        if (null == taCustomer) {
//            taCustomer = newCustomerByPerson(taPerson);
        }

        // 新增前台记录
//        iTaMainUsherRecordService.newRecordByDeviceDeprecated(taCustomer, taPerson);
        iTaMainUsherRecordService.newRecordByDevice(taPerson, taCustomer);
        return;
    }

    @Override
    public void deleteCustOfPerson(Integer personId, Integer realId) {
        // 删除映射表
        QueryWrapper<TaCustomerMap> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("person_id", personId).or().eq("person_id", realId);
        taCustomerMapMapper.delete(queryWrapper);

        // 删除客户表, 更新状态为删除
        // 默认客户ID 与人员ID是一致的
        UpdateWrapper<TaCustomer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Delete);
        updateWrapper.eq("customer_id", personId).or().eq("customer_id", realId);
        taCustomerMapper.update(new TaCustomer(), updateWrapper);
    }

    @Override
    public List<TaCustomer> getSomeCustMayBe(String plateNumber) {
        return taCustomerMapper.getSomeCustMayBe(plateNumber, null);
    }

    @Override
    public TaCustomer newCustByCarDevice(TaCarRecord taCarRecord, TaVisitorAppointment ...taVisitorAppointments) {
        TaCustomer taCustomer = new TaCustomer();
        TaVisitorAppointment appointment = null != taVisitorAppointments && taVisitorAppointments.length > 0 ? taVisitorAppointments[0] : null;

        String name = StringUtils.ifNull(taCarRecord.getCustomerName(), null != appointment ? appointment.getPersonName() : null);
        String phone = null != appointment ? appointment.getPhone() : null;
        Integer consultantId = null != appointment ? appointment.getCreateUser() : null;

        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCustomerName(name);
        taCustomer.setPhone(phone);
        taCustomer.setAvatar(taCarRecord.getDriverImage());
        taCustomer.setCarModel(taCarRecord.getCarModel());
        taCustomer.setPlateNumber(taCarRecord.getPlateNumber());
        taCustomer.setConsultantId(consultantId);

        TaPerson taPerson = iTaPersonService.newPersonByFront(taCustomer);
        taCustomer.setCustomerId(taPerson.getRealId());

        TaCustomerMap customerMap = new TaCustomerMap();
        customerMap.setCustomerId(taCustomer.getCustomerId());
        customerMap.setPersonId(taPerson.getPersonId());

        taCustomerMapMapper.insert(customerMap);

        insertCustomer(taCustomer);

        return taCustomer;
    }

    @Override
    public void notifyConsultant(TaCustomer taCustomer) throws Exception {
        String tpl = miniApp.getTpls().getVisitor();
        String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + taCustomer.getCustomerId();
        String openid = taWechatUserMapper.getOpenidOf(taCustomer.getConsultantId(), CommConstant.WECHAT_CLIENT_MINIAPP);
        if (null == openid) {
            throw new Exception("发送消息给置业失败, 当前置业[" + taCustomer.getConsultantId() + "]未绑定小程序账号");
        }
        List<Object> data = new ArrayList<Object>(){{
            add(taCustomer.getCustomerName());
            add(taCustomer.getPhone());
            add("您有客户来访, 请及时接待!");
        }};

        miniApp.sendTPLMessage(tpl, openid, page, data);
    }

    @Override
    public boolean updateConsultant(TaCustomer taCustomer, Integer consultantId) {
        SysUser usr = sysUserMapper.selectById(consultantId);
        if (null == usr) {
            log.error("更新客户置业失败, 置业ID {} 不存在", consultantId);
            return false;
        }

        Integer originConsultant = taCustomer.getConsultantId();
        taCustomer.setConsultantId(consultantId);
        taCustomer.setRealtyConsultant(usr.getUsername());

        // 如果变更了置业顾问
        if (null == originConsultant || originConsultant.intValue() < 1 || !originConsultant.equals(consultantId)) {
            try {
                notifyConsultant(taCustomer);
            } catch (Exception e) {}
        }

        int row = taCustomerMapper.updateById(taCustomer);
        return row > 0;
    }

    @Override
    public TaCustomer newCustByMainUsher(TaMainUsherRecord taMainUsherRecord, boolean notify) throws Exception {
        Integer personId = taMainUsherRecord.getPersonId();
        if (null == personId) {
            TaPerson taPerson = iTaPersonService.newPersonByUsher(taMainUsherRecord);
            personId = taPerson.getRealId();
            taMainUsherRecord.setPersonId(personId);
        }

        Integer custId = taMainUsherRecord.getCustomerId();
        TaCustomer cust;
        if (null == custId) {
            cust = copyDataFromMainUsher(taMainUsherRecord);
            if (!insertCustomer(cust)) {
                throw new Exception("新增客户信息失败");
            }

            taMainUsherRecord.setCustomerId(cust.getCustomerId());
        } else {
            cust = taCustomerMapper.selectById(custId);
        }

        TaCustomerMap taCustomerMap = taCustomerMapMapper.getCustomerOf(personId);
        if (null == taCustomerMap) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setCustomerId(cust.getCustomerId());
            custMap.setPersonId(taMainUsherRecord.getPersonId());

            int row = taCustomerMapMapper.insert(custMap);
            if (row <= 0) {
                throw new Exception("生成置业客户映射失败");
            }
        }

        // 发消息给置业
        if (null != cust.getConsultantId() && notify) {
            notifyConsultant(cust);
        }

        return cust;
    }

    @Override
    public Map<String, Object> getHistoryRecBy(Map<String, Object> filters) {
        Map<String, Object> result = new HashMap<String, Object>(){{
            put("phone", filters.get("phone").toString());
        }};

        List<TaVisitorAppointment> appointments = iTaVisitorAppointmentService.getAppointmentsBy(filters);
        if (null != appointments && appointments.size() > 0) {
            result.put("source", "appointment");
            result.put("appointment", 1);
            result.put("name", appointments.get(0).getPersonName());
            result.put("consultantId", appointments.get(0).getCreateUser());
            result.put("sex", appointments.get(0).getSex());
            result.put("plateNumber", appointments.get(0).getPlatNumber());
            result.put("customerId", "");
        }

        List<TaCustomer> custs = taCustomerMapper.getSomeCustMayBe(null, filters.get("phone").toString());
        if (null != custs && custs.size() > 0) {
            result.put("source", "customer");
            result.put("name", custs.get(0).getCustomerName());
            result.put("consultantId", custs.get(0).getConsultantId());
            result.put("consultantName", custs.get(0).getRealtyConsultant());
            result.put("purpose", custs.get(0).getPurpose());
            result.put("sex", custs.get(0).getSex());
            result.put("plateNumber", custs.get(0).getPlateNumber());
            result.put("customerId", custs.get(0).getCustomerId());
        }

        return result;
    }

    @Override
    public TaCustomer getSameByPerson(Integer personId) {
        return taCustomerMapper.getCustomerOf(personId, personId);
    }

    @Override
    public boolean updataByCarRecord(TaCustomer taCustomer, TaCarRecord taCarRecord) {
        boolean needUpate = false;

        if (StringUtils.isEmpty(taCustomer.getPlateNumber())) {
            needUpate = true;
            taCustomer.setPlateNumber(taCarRecord.getPlateNumber());
            taCustomer.setCarModel(taCarRecord.getCarModel());
        }

        if (needUpate) {
            return taCustomerMapper.updateById(taCustomer) > 0;
        }

        return true;
    }

    @Override
    public TaCustomer getByPhoneOrPlate(String phone, String plateNumber) {
        QueryWrapper<TaCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(phone != null,"phone", phone).or().eq(plateNumber != null, "plate_number", plateNumber);
        // 故意设置一个查询不到数据的条件
        queryWrapper.eq(null == phone || null == plateNumber, "customer_id", -1);
        queryWrapper.orderByDesc("status");
        queryWrapper.orderByDesc("create_date");
        queryWrapper.last("limit 1");

        return taCustomerMapper.selectOne(queryWrapper);
    }

    @Override
    public TaCustomer getWithLastVisit(Integer id) {
        TaCustomer taCustomer = taCustomerMapper.selectById(id);
        if (null == taCustomer) {
            return null;
        }
        //获取字典表所有信息
        List<SysDict> dictList = iSysDictService.list();
        dictList.stream().forEach(e -> {
            //对比数据重新赛值
            if (e.getId().equals(taCustomer.getNowArea())){
                taCustomer.setNowAreaLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getFamilyWork())){
                taCustomer.setFamilyWorkLable(e.getLabel());
            }
            if (e.getId().equals(Integer.valueOf(taCustomer.getFamilyIndustry()))){
                taCustomer.setFamilyIndustryLable(e.getLabel());
            }
            if (e.getId().equals(Integer.valueOf(taCustomer.getFamilyDuty()))){
                taCustomer.setFamilyDutyLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getHousePurchasing())){
                taCustomer.setHousePurchasingLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getHouseProperty())){
                taCustomer.setHousePropertyLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getHouseLivingArea())){
                taCustomer.setHouseLivingAreaLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getHouseType())){
                taCustomer.setHouseTypeLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getFamilyStructure())){
                taCustomer.setFamilyStructureLable(e.getLabel());
            }
            if (e.getId().equals(taCustomer.getCustomerAge())){
                taCustomer.setCustomerAgeLable(e.getLabel());
            }
            if (e.getId().equals(Integer.valueOf(taCustomer.getFavorLevel()))){
                taCustomer.setFavorLevelLable(e.getLabel());
            }
        });

        TaMainUsherRecord taMainUsherRecord = iTaMainUsherRecordService.getTodayRecord(taCustomer.getCustomerId());
        if (null != taMainUsherRecord) {
            List<TaFollowupRecord> follows = iTaFollowupRecordService.getByMainRecord(taMainUsherRecord.getRecId());

            if (null != follows && follows.size() > 0) {
                Map<String, Object> lastVisit = new HashMap<String, Object>(){{
                    put("follows", follows);
                }};
                taCustomer.setLastVisit(lastVisit);
            }
        }

        return taCustomer;
    }

    @Override
    public IPage<TaCustomer> getCustomerListOfFront(int pageNumber, int pageSize, Integer consultantId, String nameOrPhone) {
        IPage<TaCustomer> page = new Page<>(pageNumber, pageSize);
        return taCustomerMapper.getCustomerListOfFront(page, consultantId, nameOrPhone);
    }

    private TaCustomer newCustomerByPerson(TaPerson taPerson) throws Exception {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taPerson.getRealId());
        taCustomer.setAvatar(taPerson.getAvatar());
        taCustomer.setPhone(taPerson.getPhone());
//            taCustomer.setSex(taPerson.getSex());
        taCustomer.setCustomerName(taPerson.getName());
        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setStatus(StatusUtils.Normal);

        if (!insertCustomer(taCustomer)) {
            throw new Exception("新增客户失败");
        }

        int mapsNum = taPerson.getPersonId().equals(taPerson.getRealId()) ? 1 : 2;

        for (int i = 0; i < mapsNum; i ++) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setPersonId(i == 0 ? taPerson.getRealId() : taPerson.getPersonId());
            custMap.setCustomerId(taCustomer.getCustomerId());
            int row = taCustomerMapMapper.insert(custMap);
            if (row < 1) {
                throw new Exception("新增客户映射信息失败");
            }
        }

        return taCustomer;
    }

    private TaCustomer copyDataFromMainUsher(TaMainUsherRecord taMainUsherRecord) {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taMainUsherRecord.getPersonId());       // 默认客户ID与人员ID一致
        taCustomer.setCustomerName(taMainUsherRecord.getPersonName());
        taCustomer.setSex(taMainUsherRecord.getSex());
        taCustomer.setPhone(taMainUsherRecord.getPhone());
        taCustomer.setAvatar(taMainUsherRecord.getAvatar());
        taCustomer.setPlateNumber(taMainUsherRecord.getPlateNumber());
        taCustomer.setCarModel(taMainUsherRecord.getCarModel());
        taCustomer.setChannel(taMainUsherRecord.getChannel());
        taCustomer.setPurpose(taMainUsherRecord.getPurpose());
        taCustomer.setConsultantId(taMainUsherRecord.getConsultantId());
        taCustomer.setRealtyConsultant(taMainUsherRecord.getRealtyConsultant());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCreateDate(LocalDateTime.now());

        return taCustomer;
    }

    private boolean insertCustomer(TaCustomer taCustomer) {
        Integer consultantId = taCustomer.getConsultantId();
        if (null != consultantId && consultantId.intValue() > 0) {
            try {
                notifyConsultant(taCustomer);
            } catch (Exception e) {}
        }

        return taCustomerMapper.insert(taCustomer) > 0;
    }

}
