package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.TaCustomerMapper;
import com.huiju.welcome.mapper.TaMainUsherRecordMapper;
import com.huiju.welcome.mapper.TaTransactionRecordMapper;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaMainUsherRecord;
import com.huiju.welcome.model.TaTransactionRecord;
import com.huiju.welcome.service.ITaTransactionRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.Period;
import java.util.List;

/**
 * <p>
 * 成交记录表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-07-17
 */
@Service
public class TaTransactionRecordServiceImpl extends ServiceImpl<TaTransactionRecordMapper, TaTransactionRecord> implements ITaTransactionRecordService {
	
	@Autowired
	private TaTransactionRecordMapper taTransactionRecordMapper;
	
	@Autowired
	private TaCustomerMapper taCustomerMapper;
	
	@Autowired
	private TaMainUsherRecordMapper taMainUsherRecordMapper;
	
	/**
	 * 更具客户id查询数据没有的情况下在查一些预置数据
	 *
	 * @param customerId
	 * @return
	 */
	@Override
	public TaTransactionRecord getRecordByCustomerId(Integer customerId) {
		QueryWrapper<TaTransactionRecord> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("customer_id",customerId);
		TaTransactionRecord taTransactionRecord = taTransactionRecordMapper.selectOne(queryWrapper);
		//没有成交记录的情况下自己构造数据
		if (null == taTransactionRecord){
			TaTransactionRecord transactionRecord = new TaTransactionRecord();
			//查询客户信息
			TaCustomer taCustomer = taCustomerMapper.selectById(customerId);
			transactionRecord.setCustomerId(customerId);
			transactionRecord.setCustomerName(taCustomer.getCustomerName());
			transactionRecord.setCustomerIdcard(taCustomer.getIdcard());
			//查询到访记录信息
			QueryWrapper<TaMainUsherRecord> taMainUsherRecordQueryWrapper = new QueryWrapper<>();
			taMainUsherRecordQueryWrapper.eq("customer_id",customerId);
			taMainUsherRecordQueryWrapper.eq("status","1");
			taMainUsherRecordQueryWrapper.orderByAsc("visite_date");
			List<TaMainUsherRecord> taMainUsherRecords = taMainUsherRecordMapper.selectList(taMainUsherRecordQueryWrapper);
			//计算到访时间和到访次数
			if (taMainUsherRecords.size() == 0){
				transactionRecord.setVisitingRecord("0");
				transactionRecord.setTransactionCycle("0");
			}else if (taMainUsherRecords.size() == 1){
				transactionRecord.setVisitingRecord(String.valueOf(taMainUsherRecords.size()));
				transactionRecord.setTransactionCycle("1");
			}else {
				transactionRecord.setVisitingRecord(String.valueOf(taMainUsherRecords.size()));
				LocalDateTime startDate = taMainUsherRecords.get(0).getVisiteDate();
				LocalDateTime endDate = taMainUsherRecords.get(taMainUsherRecords.size() - 1).getVisiteDate();
				int daysNum = Period.between(startDate.toLocalDate(), endDate.toLocalDate()).getDays();
				transactionRecord.setTransactionCycle(String.valueOf(daysNum));
			}
			return transactionRecord;
		}
		return taTransactionRecord;
	}
}
